/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: BuildSingles                                       *
*                                                              *
*   Using lists of splice sites, it builds single genes        *
***************************************************************/

#include "geneid.h"

long BuildSingles(site *Start, long nStarts, 
		  site *Stop, long nStops,
		  long cutPoint,
		  exonGFF *Exon) 
{
  int Frame;
  long i, j, js;
  long nSingles;
  
  /* main loop, for each Start codon */
  for (i=0, j=0, nSingles=0;
      (i < nStarts) && (j<nStops) &&
      ((Start+i)->Position < STARTOATG) &&
      (nSingles<NUMEXONS);
      i++)
    {
      Frame = (Start+i)->Position % 3;
   
      /* advance Stops to Start */
      while ( (j < nStops) && 
	      (((Stop+j)->Position+1) < (Start+i)->Position))
	j++;

      js=j;

      /* Only use some stops (stops before cutPoint) */
      /* hacia este bucle con la j dejandola tras el cutpoint */
/*        while ( (js < nStops) &&  */
/*  	      (((Stop+j)->Position) <= cutPoint)) */
/*  	js++; */

      /* find first Stop in Frame with Start: close its frame */
      while ((js < nStops) && (((Stop+js)->Position+1) % 3 != Frame))
	js++;

      /* Only use some stops (stops before cutPoint) */
      /* first Stop after Start defines a single gen */
      if (js < nStops && (Stop+js)->Position >= cutPoint)
	{
	  /* Length rule about Single Genes */
	  if ( ((Stop+js)->Position + LENGTHCODON - (Start+i)->Position + 1)
	       >= 
	       SINGLEGENELENGTH)
	    {
	      (Exon + nSingles)->Acceptor = (Start+i);
	      (Exon + nSingles)->Donor = (Stop+js);
	      (Exon + nSingles)->Frame = 0;
	      (Exon + nSingles)->Remainder = 0;
	      strcpy((Exon + nSingles)->Type,"Single");
         (Exon + nSingles)->Group = 0;
         (Exon + nSingles)->evidence = 0;

	      nSingles++;
	    }
	}
    }

  if (nSingles==NUMEXONS)
   printError("Too many predicted exons: Change NUMEXONS parameter");

  return(nSingles);
}
