/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: BuildTGA/GTExons                                   *
*                                                              *
*   Using lists of splice sites, it builds initial exons       *
***************************************************************/


#include "geneid.h"

long BuildTGAxGT(site *PseudoStop, long nPseudoStops, 
		 site *Donor, long nDonors,
		 site *Stop, long nStops,
		 int MaxDonors, exonGFF *Exon) 
{
  exonGFF *LocalExon;
  int nLocalExons, LowestLocalExon;
  double LowestLocalScore;
  int Frame;
  long i, j=0, js, k=0, ks;
  int l;
  long nExon;
  
  LocalExon = (exonGFF*) calloc(MaxDonors, sizeof(exonGFF));
  
  /* main loop, for each TGA */
  for (i = 0, nExon = 0;
      (nExon<NUMEXONS) && (i < nPseudoStops);
      i++)
  { 
    Frame = (PseudoStop+i)->Position % 3;

    nLocalExons = 0;
    LowestLocalScore = DBL_MAX;
        
    /* advance Stops to PSeudoStop */
    while (((Stop+j)->Position+1 <= (PseudoStop+i)->Position-3) && (j < nStops))
      j++;
    js=j;
    
    /* find first Stop in Frame with Start */
    while ((((Stop+js)->Position+1) % 3 != Frame) && (js < nStops))
      js++;

    /* advance Donors to PSeudoStop */
    while (((Donor+k)->Position < (PseudoStop+i)->Position) && (k < nDonors))
      k++;
    ks=k;
    
    /* every Donor between TGA and Stop[js] defines an TGA_GT exon */
    /* keep  only the top scoring MaxDonors */

    while ((js == nStops || (Donor+ks)->Position < (Stop+js)->Position + 1 + 2) 
	   && (ks < nDonors)) 
      {
	if (nLocalExons < MaxDonors) 
	  {
	    (LocalExon+nLocalExons)->Acceptor=(PseudoStop+i);
	    (LocalExon+nLocalExons)->Donor=(Donor+ks);
	    if ((Donor+ks)->Score < LowestLocalScore) 
	      {
		LowestLocalScore = (Donor+ks)->Score;
		LowestLocalExon = nLocalExons;
	      }
	    nLocalExons++;
	  }
	else 
	  {
	    if ((Donor+ks)->Score > LowestLocalScore) {
	      for (l=LowestLocalExon;l<nLocalExons-1;l++)
		LocalExon[l]=LocalExon[l+1];
	    (LocalExon+nLocalExons-1)->Acceptor=(PseudoStop+i);
	    (LocalExon+nLocalExons-1)->Donor=(Donor+ks);
	    
	    LowestLocalExon = 0;
	    LowestLocalScore = (LocalExon+0)->Donor->Score;
	    for (l=1;l<nLocalExons;l++)
	      if ((LocalExon+l)->Donor->Score < LowestLocalScore) 
		{
		  LowestLocalScore = (LocalExon+l)->Donor->Score;
		  LowestLocalExon = l;
		}
	    }
	  }
	ks++;
      }
    
    /* put LocalExon into TGA_GT Exons. Within TGA sort by Donor */  
    for (l=0;(l<nLocalExons) && (nExon<NUMEXONS);l++) 
      {
	Exon[nExon] = LocalExon[l];
	(Exon+nExon)->Frame = 0;
	(Exon+nExon)->Remainder = ((Exon+nExon)->Donor->Position -
				   (Exon+nExon)->Acceptor->Position + 1 ) % 3;
	(Exon+nExon)->Remainder = (3 - (Exon+nExon)->Remainder) % 3; 
	strcpy((Exon+nExon)->Type,"TGA_GT");
	(Exon+nExon)->Group = 0;
	(Exon+nExon)->evidence = 0;
	nExon++;
    }
  }

  if (nExon==NUMEXONS)
   printError("Too many predicted exons: Change NUMEXONS parameter");

  free(LocalExon);
  return(nExon);
}
