/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: BuildTGAxSTOP.c                                    *
*                                                              *
*   Using lists of splice sites, it builds TGAxSTOP exons      *
***************************************************************/

#include "geneid.h"

long BuildTGAxSTOP(site *PseudoStop, long nPseudoStops,
		  site *Stop, long nStops,
		  long cutPoint,
		  exonGFF *Exon) 
{
  int Frame;
  long i, j, js;
  long nTGAxSTOPs;
  
  /* main loop, for each TGA */
  for (i=0, j=0, nTGAxSTOPs=0;
      (i < nPseudoStops) && (j<nStops) &&
      (nTGAxSTOPs<NUMEXONS);
      i++)
    {
      Frame = (PseudoStop+i)->Position % 3;
   
      /* advance Stops to PseudoStop */
      while ( (j < nStops) && 
	      (((Stop+j)->Position+1) <= (PseudoStop+i)->Position-3))
	j++;

      js=j;

      /* Only use some stops (stops before cutPoint) */
      /* find first Stop in Frame with PseudoStop: close its frame */
      while ((js < nStops) && (((Stop+js)->Position+1) % 3 != Frame))
	js++;

      /* Only use some stops (stops before cutPoint) */
      /* first Stop after PseudoStop defines a TGAxSTOP gen */
      if (js < nStops && (Stop+js)->Position >= cutPoint)
	{
	  (Exon + nTGAxSTOPs)->Acceptor = (PseudoStop+i);
	  (Exon + nTGAxSTOPs)->Donor = (Stop+js);
	  (Exon + nTGAxSTOPs)->Frame = 0;
	  (Exon + nTGAxSTOPs)->Remainder = 0;
	  strcpy((Exon + nTGAxSTOPs)->Type,"TGA_STOP");
	  (Exon + nTGAxSTOPs)->Group = 0;
	  (Exon + nTGAxSTOPs)->evidence = 0;
	  
	  nTGAxSTOPs++;
	}
    }

  if (nTGAxSTOPs==NUMEXONS)
   printError("Too many predicted exons: Change NUMEXONS parameter");

  return(nTGAxSTOPs);
}
