/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: BuildTerminalExons                                 *
*                                                              *
*   Using lists of splice sites, it builds terminal exons      *
***************************************************************/

#include "geneid.h"

long BuildTerminalExons (site *Acceptor, long nAcceptors, 
		         site *Stop, long nStops,
			 long LengthSequence,
			 exonGFF* Exon,
			 long cutPoint)
{
  int Frame[3];
  long StartSearch;
  long iA=0,
       i, f,
       j=0, js;

  long nExon=0;

  /* advance Acceptor and Stop to SeqLength - STOPTOEND */
  StartSearch = LengthSequence - STOPTOEND;
  while((Acceptor+iA)->Position < StartSearch)
    iA++;
  
  while((Stop+j)->Position+1 < StartSearch)
    j++;
  
  /* main loop, for each Acceptor next Stop in every Frame defines an exon */
  for (i=iA;(i<nAcceptors) && (nExon<NUMEXONS);i++) 
    {
      /* Open the windows... */
      for (f=0;f<3;f++)
	Frame[f]=1;

      /* advance Stops to Acceptor */
      while (((Stop+j)->Position+1 < (Acceptor+i)->Position) && (j < nStops))
	j++;
      js=j;
      
      /* Find Stops in Frame still open */
      while ((Frame[0]==1 || Frame[1]==1 || Frame[2]==1)
               && (js < nStops)
               && (nExon<NUMEXONS))
	{
	  if (Frame[f=((Stop+js)->Position - (Acceptor+i)->Position + 1) % 3])
	    {
	      if ((Stop+js)->Position >= (Acceptor+i)->Position &&
		  (Stop+js)->Position >= cutPoint)
		{
		  (Exon+nExon)->Acceptor=(Acceptor+i);
		  (Exon+nExon)->Donor=(Stop+js);
		  (Exon+nExon)->Frame = f;
		  (Exon+nExon)->Remainder = 0; 
		  strcpy((Exon+nExon)->Type,"Terminal");
        (Exon+nExon)->Group = 0;
        (Exon+nExon)->evidence = 0;
		  nExon++;
		}
	      Frame[f]=0;
       } 
	  js++;
   }     
    }    

  if (nExon==NUMEXONS)
   printError("Too many predicted exons: Change NUMEXONS parameter");

  return(nExon);
}
