/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: GetSitesWithProfile                                *
*                                                              *
*   Splice sites prediction using Position Frequency Matrices  *
***************************************************************/

#include "geneid.h"

long  GetSitesWithProfile(char *s, profile *p, site *st, 
			  long l1, long l2) 
{
  int i;
  double score;
  long ns=0,
       is=0;
  long left, 
       right;
 
  /* left and right are the true boundaries of prediction */
  left  = MAX(0,l1 - p->offset);
  right = l2 - p->offset;
  s += left;

  /* discovering splice sites with current profile */
  while (*(s+p->dimension) && (is < right- left + 1) && (ns<NUMSITES))
    {
      /* is = 0..right */
      score=0.0;
      for (i=0;i<p->dimension;i++) 
	score=score+p->matrix[(int)(*(s+i))][i];
      if (score >= p->cutoff) 
	{
	  st[ns].Position = left + is + p->offset;
	  st[ns].Score=score;
	  ns++;
	}
      is++;
      s++;
    }

  if (ns==NUMSITES)
   printError("Too many predicted sites: Change NUMSITES parameter");

  return(ns);
}

 
  
