/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: GetSitesWithProfile                                *
*                                                              *
*   Stop codons prediction using Position Frequency Matrices   *
***************************************************************/

#include "geneid.h"

long GetStopCodons(char *s, profile *p, site *sc, long l1, long l2) 
{
  long ns=0,
       is=0;
  double score;
  int i;
  
  static char *stop[] = 
  {
    "TAA",
    "TAG",
    "TGA"
  };

  char codon[LENGTHCODON];
  long left, 
       right;

  /* left and right are the true boundaries of prediction */
  left  = MAX(0,l1 - p->offset);
  right = l2 - p->offset;
  s += left;

  /* discovering splice sites with current profile */
  while (*(s+p->dimension) && (is < right - left + 1) && (ns<NUMSITES))  
    {
      strncpy(codon,s+p->offset+1,LENGTHCODON); 
      codon[LENGTHCODON]='\0';
      
      if (!strcmp(codon,stop[0]) ||
          !strcmp(codon,stop[1]) ||
          !strcmp(codon,stop[2])) 
	{
	  score=0.0;
	  for (i=0;i<p->dimension;i++) 
	    score=score+p->matrix[(int)(*(s+i))][i];

	  if (score >= p->cutoff) 
	    {
	      /* Position given is last coding position before Stop */
	      sc[ns].Position=left + is + p->offset;
/*  	      sc[ns].Score=score; */
	      sc[ns].Score=0;
	      ns++;
	    }
	} 
      is++;
      s++;
    }

  /* Get remaining stops at end of sequence. Set score to 0 for those */
  if (!(s+p->dimension))
    {
      s=(s-is);
      is+=p->offset;

      while (*(s+is) && (ns<NUMSITES))  
	{
	  strncpy(codon,(s+is),3); 
	  codon[3]='\0';
	  if (!strcmp(codon,stop[0]) || !strcmp(codon,stop[1]) || 
	      !strcmp(codon,stop[2])) 
	    {
         /* Position given is last coding position before Stop */
         sc[ns].Position=is-1;  
	      sc[ns].Score=0;
	      ns++;
	    } 
	  is++;
	}
    }

  if (ns==NUMSITES)
   printError("Too many predicted sites: Change NUMSITES parameter");

  return(ns);
}
