/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: Output                                             *
*                                                              *
*   Management of options for printing results                 *
***************************************************************/


#include "geneid.h"

extern int VRB;

extern int SFP, SDP, SAP, STP, SPP,
  EFP, EIP, ETP, EXP, ESP,EJP,EKP,
           FWD, RVS,
           GENAMIC, GENEID,
           GFF;


extern account *m;
        
/* Printing messages (information) */
void printMess(char* s)
{
  if (VRB)
     fprintf (stderr, "> %s\n",s);
}

/* Printing partial and final results (information) */
void printRes(char* s)
{
  if (VRB)
     fprintf (stderr, "\t%s\n",s);
}

/* Printing error messages */
void printError(char *s)
{
     fprintf(stderr,"Error: %s\n",s);
	  exit(1);
}

void PrintProfile (FILE *f, profile *p)
{
  int i;

  if (VRB)
  {
  fprintf(f, "%d %d %5.3f\n", p->dimension, p->offset, p->cutoff);
  
  for (i=0;i<p->dimension;i++) 
    fprintf (f, " %9.3f", p->matrix['A'][i]);
  fprintf(f, "\n");

  for (i=0;i<p->dimension;i++) 
    fprintf (f, " %9.3f", p->matrix['C'][i]);
  fprintf(f, "\n");

  for (i=0;i<p->dimension;i++) 
    fprintf (f, " %9.3f", p->matrix['G'][i]);
  fprintf(f, "\n");

  for (i=0;i<p->dimension;i++) 
    fprintf (f, " %9.3f", p->matrix['T'][i]);
  fprintf(f, "\n\n");
  }
}

void OutputHeader()
{
  char* s;
  
  if (GFF)
    printf("## gff-version 2\n");
  
  printf("## source-version: GeneId v 3.0 EvoI -- geneid@imim.es\n");
  s = ctime(&m->tStart);

  printf("## date %s",s);
}

/* Print some predictions results according to the options selected */
void Output(packSites* allSites, packSites* allSites_r,
	    packExons* allExons, packExons* allExons_r,
	    exonGFF* exons, long nExons, char* Locus, 
       long l1, long l2, char* Sequence, gparam* gp, dict* dAA)
{
  /* 1. Printing Forward */
  if (FWD)
    {
      printMess("Printing forward selected elements");
	    
      /* sites */
      if (SFP) 
	PrintSites(allSites->StartCodons, allSites->nStartCodons,
		   STA, Locus, FORWARD, l1, l2, Sequence, gp->StartProfile);
      if (SAP)
	PrintSites(allSites->AcceptorSites, allSites->nAcceptorSites,
		   ACC, Locus, FORWARD, l1, l2, Sequence, gp->AcceptorProfile);
      if (SDP)
	PrintSites(allSites->DonorSites, allSites->nDonorSites,
		   DON, Locus, FORWARD, l1, l2, Sequence, gp->DonorProfile);
      if (STP)
	PrintSites(allSites->StopCodons, allSites->nStopCodons,
		   STO, Locus, FORWARD, l1, l2, Sequence, gp->StopProfile);
      if (SPP)
	PrintSites(allSites->PseudoStops, allSites->nPseudoStops,
		   PSE, Locus, FORWARD, l1, l2, Sequence, gp->PseudoProfile);
      
      /* exons */
      if (EFP)
	PrintExons(allExons->InitialExons,allExons->nInitialExons,
		   FIRST, Locus, l1, l2, Sequence, dAA);
      if (EIP) 
	PrintExons(allExons->InternalExons,allExons->nInternalExons,
		   INTERNAL, Locus, l1, l2, Sequence, dAA);
      if (ETP)
	PrintExons(allExons->TerminalExons,allExons->nTerminalExons,
		   TERMINAL, Locus, l1, l2, Sequence, dAA);
      if (ESP)
	PrintExons(allExons->Singles,allExons->nSingles,
		   SINGLE, Locus, l1, l2, Sequence, dAA);
      if (EJP)
	PrintExons(allExons->TGAxGT,allExons->nTGAxGTs,
		   TGA_GT,Locus, l1, l2, Sequence, dAA);
      if (EKP)
	PrintExons(allExons->TGAxSTOP,allExons->nTGAxSTOPs,
		   TGA_STOP,Locus, l1, l2, Sequence, dAA);
    }

  /* 2. Printing Reverse */
  if (RVS)
    {
      printMess("Printing reverse selected elements");
   
      /* sites */ 
      if (SFP)
	PrintSites(allSites_r->StartCodons,allSites_r->nStartCodons,STA,
		   Locus,REVERSE, l1, l2, Sequence, gp->StartProfile);
      if (SAP)      
	PrintSites(allSites_r->AcceptorSites,allSites_r->nAcceptorSites,ACC,
		   Locus,REVERSE, l1, l2, Sequence, gp->AcceptorProfile);
      if (SDP)      
	PrintSites(allSites_r->DonorSites,allSites_r->nDonorSites,DON,
		   Locus,REVERSE, l1, l2, Sequence, gp->DonorProfile);
      if (STP)
	PrintSites(allSites_r->StopCodons,allSites_r->nStopCodons,STO,
		   Locus,REVERSE, l1, l2, Sequence, gp->StopProfile);
      if (SPP)
	PrintSites(allSites_r->PseudoStops, allSites_r->nPseudoStops,PSE,
		   Locus, REVERSE, l1, l2, Sequence, gp->PseudoProfile);
      /* exons */
      if (EFP)
	PrintExons(allExons_r->InitialExons,allExons_r->nInitialExons,
		   FIRST, Locus,  l1, l2, Sequence, dAA);
      
      if (EIP) 
	PrintExons(allExons_r->InternalExons,allExons_r->nInternalExons,
		   INTERNAL, Locus,  l1, l2, Sequence, dAA);
      if (ETP) 
	PrintExons(allExons_r->TerminalExons,allExons_r->nTerminalExons,
		   TERMINAL, Locus,  l1, l2, Sequence, dAA);
      if (ESP)
	PrintExons(allExons_r->Singles,allExons_r->nSingles,
		   SINGLE, Locus, l1, l2, Sequence, dAA);
      if (EJP)
	PrintExons(allExons_r->TGAxGT,allExons_r->nTGAxGTs,
		   TGA_GT,Locus, l1, l2, Sequence, dAA);
      if (EKP)
	PrintExons(allExons_r->TGAxSTOP,allExons_r->nTGAxSTOPs,
		   TGA_STOP,Locus, l1, l2, Sequence, dAA);

    }
  
  /* 3. Print all exons */
  if (EXP) 
    {
      printMess("Printing all predicted Exons of current split");   
      PrintExons(exons, nExons, FIRST + INTERNAL + TERMINAL + SINGLE, 
		 Locus, l1, l2, Sequence, dAA);
    }
}

/* Print genes using selected format */
void OutputGene(packGenes* pg, long nExons, char* Locus,
                char* Sequence, gparam* gp, dict* dAA)
{
  /* Recover the Best Gene recursively */
  if (nExons>0)
    {
      printMess("Recovering gene-solution...");
      CookingGenes(pg->GOptim, Locus, Sequence, gp, dAA);
    }
}

void OutputStats(char* Locus, long nvExons)
{
  char mess[MAXSTRING];

  if (GENEID)
    {
      sprintf(mess,"\n\tStats (Sequence %s)",Locus);
      printRes(mess);
      
/*        printRes("_______________________________________________________________\n"); */
/*        sprintf(mess,"%8s\t%8s\t%8s\t%8s", */
/*  	      sSTA,sACC,sDON,sSTO); */

/*        printRes(mess); */
/*        printRes("_______________________________________________________________\n"); */
      
/*        sprintf(mess,"%8ld\t%8ld\t%8ld\t%8ld\n", */
/*  	      m->starts, */
/*  	      m->acc, */
/*  	      m->don, */
/*  	      m->stops); */

/*        printRes(mess); */
      
/*        sprintf(mess,"%8ld\t%8ld\t%8ld\t%8ld\n", */
/*  	      m->starts_r, */
/*  	      m->acc_r, */
/*  	      m->don_r, */
/*  	      m->stops_r); */

/*        printRes(mess); */
      
      printRes("_______________________________________________________________\n");

      sprintf(mess,"%8s\t%8s\t%8s\t%8s",
	      sFIRST,sINTERNAL,sTERMINAL,sSINGLE);
      
      printRes(mess);
      printRes("_______________________________________________________________\n");

      sprintf(mess,"%8ld\t%8ld\t%8ld\t%8ld\n",      
	      m->first, 
	      m->internal, 
	      m->terminal,
	      m->single); 
      printRes(mess);

      sprintf(mess,"%8ld\t%8ld\t%8ld\t%8ld\n",      
	      m->first_r, 
	      m->internal_r, 
	      m->terminal_r,
	      m->single_r);      
      printRes(mess);

      sprintf(mess,"TOTAL: %ld predicted exons (%ld evidence exons)\n",
	      m->totalExons,
	      nvExons);
      printRes(mess);
    }
}

/* Get time of execution using account information */
void OutputTime()
{
  time_t tEnd;
  int t;
  float caux;
  char mess[MAXSTRING];

  (void) time(&tEnd);
  caux = (float)clock() / (float)CLOCKS_PER_SEC;

  t = (int) tEnd - m->tStart;

  if (t < caux)
    t++;

  printRes("_______________________________________________________________\n");
 
  sprintf(mess,"CPU time: \t%.3f secs",caux);
  printRes(mess);

  sprintf(mess,"Total time: \t%d secs(%.2f mins)\n\n",t,(float)t/MINUTE);
  printRes(mess);
}



