/*************************************************************************
*                                                                        *
*   Module: ReadExonsGFF                                                 *
*                                                                        *
*   GenAmic alone: It reads exons in GFF format.                         *
*                                                                        *
*   This file is part of the geneid Distribution                         *
*                                                                        *
*     Copyright (C) 2000 - Enrique BLANCO GARCIA                         *
*                          Roderic GUIGO SERRA                           * 
*                                                                        *
*  This program is free software; you can redistribute it and/or modify  *
*  it under the terms of the GNU General Public License as published by  *
*  the Free Software Foundation; either version 2 of the License, or     *
*  (at your option) any later version.                                   *
*                                                                        *
*  This program is distributed in the hope that it will be useful,       *
*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*  GNU General Public License for more details.                          *
*                                                                        *
*  You should have received a copy of the GNU General Public License     *
*  along with this program; if not, write to the Free Software           * 
*  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.             *
*************************************************************************/

#include "geneid.h"

long ReadExonsGFF (char *FileName, packEvidence* pv, dict* d)
{
  long i;
  FILE *file;
  char line[MAXLINE];
  char saux[MAXTYPE+1];
  short fraux;
  char c1,c2;
  int three;
  long lastAcceptor;
  long currAcceptor;
  char mess[MAXSTRING];
 
  if ((file=fopen(FileName, "r"))==NULL)
    printError("The exonsGFF file cannot be open for read");
  
  /* Coments: line begins with # */
  /* gff format = "Name  Source  Type  Begin  End  Score  Strand  Frame */
  i = 0; 
  three = 0; 
  lastAcceptor = -INFI;
  while(fgets(line,MAXLINE,file)!=NULL)
    {
      if(line[0]=='#')
	{
	  /* Skip this line */
	}
      else
	{
	  /* Exon with frame and score */
	  if (sscanf(line, "%*s %*s %s %ld %ld %lf %c %hd %d",
		     (pv->vExons+i)->Type,
		     &((pv->vSites + pv->nvSites)->Position),
		     &((pv->vSites + pv->nvSites + 1)->Position),
		     &((pv->vExons+i)->Score),
		     &((pv->vExons+i)->Strand),
		     &((pv->vExons+i)->Frame),
		     &((pv->vExons+i)->Group)) != 7)
	    {
	      /* Exon with frame but not score */
	      if (sscanf(line, "%*s %*s %s %ld %ld %c %c %hd %d",
			 (pv->vExons+i)->Type,
			 &((pv->vSites + pv->nvSites)->Position),
			 &((pv->vSites + pv->nvSites + 1)->Position),
			 &c1,
			 &((pv->vExons+i)->Strand),
			 &((pv->vExons+i)->Frame),
			 &((pv->vExons+i)->Group)) ==7 && (pv->vExons+i)->Strand!='.')
		(pv->vExons+i)->Score = MAXSCORE;

	      else
		{
		  /* Exon with score but not frame */
		  if (sscanf(line, "%*s %*s %s %ld %ld %lf %c %c %d",
			     (pv->vExons+i)->Type,
			     &((pv->vSites + pv->nvSites)->Position),
			     &((pv->vSites + pv->nvSites + 1)->Position),
			     &((pv->vExons+i)->Score),
			     &((pv->vExons+i)->Strand),
			     &c1,
			     &((pv->vExons+i)->Group)) == 7 && (pv->vExons+i)->Strand!='.')
		    three = 1;
		  else
		    {
		      /* Exon with not score and not frame */
		      if (sscanf(line, "%*s %*s %s %ld %ld %c %c %c %d",
				 (pv->vExons+i)->Type,
				 &((pv->vSites + pv->nvSites)->Position),
				 &((pv->vSites + pv->nvSites + 1)->Position),
				 &c1,
				 &((pv->vExons+i)->Strand),
				 &c2,
				 &((pv->vExons+i)->Group)) == 7)
			{
			  (pv->vExons+i)->Score = MAXSCORE;
			  three = 1; 
			}
		      else
			{
			  sprintf(mess, "Error reading evidence exons: line %ld\n",i);
			  printError(mess);
			}
		    }
		}
	    }
	
	  /* What is the type of this exon? */
	  saux[0]='\0';
	  strcpy (saux, (pv->vExons+i)->Type);
	  strcat (saux, &((pv->vExons+i)->Strand));
	  
	  if (getkeyDict(d,saux) == NOTFOUND)
	    {
	      /* Type of this exon does not appear in GeneModel */
	      /* Skip this exon */
	      sprintf(mess, "Skipping exon %ld(unknown type)\n",i);
	      printMess(mess); 
	    }
	  else
	    {
	      currAcceptor = (pv->vSites + pv->nvSites)->Position;
	      /* File must be ordered by acceptor positions */
	      if (lastAcceptor > currAcceptor)
		{
		  sprintf(mess,"Exons file bad ordered by acceptor position(line %ld)",i);
		  printError(mess);  
		}
	      else
		{
		  lastAcceptor = (pv->vSites + pv->nvSites)->Position;
		  
		  /* Assign fool sites to this exon */
		  (pv->vExons+i)->Acceptor = (pv->vSites + pv->nvSites);
		  (pv->vExons+i)->Donor = (pv->vSites + pv->nvSites + 1); 
	      
		  if (three)
		    {
		      /* Creating three exons(3 frames) */
		      (pv->vExons+i+1)->Acceptor = (pv->vSites + pv->nvSites);
		      (pv->vExons+i+1)->Donor = (pv->vSites + pv->nvSites + 1); 
		      (pv->vExons+i+2)->Acceptor = (pv->vSites + pv->nvSites);
		      (pv->vExons+i+2)->Donor = (pv->vSites + pv->nvSites + 1); 
		      
		      (pv->vExons+i)->Frame = 0;
		      (pv->vExons+i+1)->Frame = 1;
		      (pv->vExons+i+2)->Frame = 2;
		      
		      strcpy((pv->vExons+i+1)->Type,(pv->vExons+i)->Type);
		      strcpy((pv->vExons+i+2)->Type,(pv->vExons+i)->Type);
		      (pv->vExons+i+1)->Score = (pv->vExons+i)->Score;
		      (pv->vExons+i+2)->Score = (pv->vExons+i)->Score;
		      (pv->vExons+i+1)->Strand = (pv->vExons+i)->Strand;
		      (pv->vExons+i+2)->Strand = (pv->vExons+i)->Strand;
		      
		      /* Remainder is ... */
		      (pv->vExons+i+1)->Remainder = 
			((3 - (((pv->vExons+i+1)->Donor->Position - 
				(pv->vExons+i+1)->Acceptor->Position - 
				(pv->vExons+i+1)->Frame + 1)%3)) %3);
		      
		      /* Remainder is ... */
		      (pv->vExons+i+2)->Remainder = 
			((3 - (((pv->vExons+i+2)->Donor->Position - 
				(pv->vExons+i+2)->Acceptor->Position - 
				(pv->vExons+i+2)->Frame + 1)%3)) %3);
		      
		      /* The same group */
		      (pv->vExons+i+1)->Group = (pv->vExons+i)->Group;
		      (pv->vExons+i+2)->Group = (pv->vExons+i)->Group;  
		      
		      /* Evidence exons */
		      (pv->vExons+i+1)->evidence = 1;
		      (pv->vExons+i+2)->evidence = 1;
		      
		      /* If strand (-), frame and remainder must be exchanged */
		      if ((pv->vExons+i)->Strand =='-')  
			{ 
			  fraux = (pv->vExons+i+1)->Frame; 
			  (pv->vExons+i+1)->Frame = (pv->vExons+i+1)->Remainder;
			  (pv->vExons+i+1)->Remainder = fraux;
			  fraux = (pv->vExons+i+2)->Frame; 
			  (pv->vExons+i+2)->Frame = (pv->vExons+i+2)->Remainder;
			  (pv->vExons+i+2)->Remainder = fraux;
			}
		    }      
		  
		  /* Remainder is ... */
		  (pv->vExons+i)->Remainder = 
		    ((3 - (((pv->vExons+i)->Donor->Position - 
			    (pv->vExons+i)->Acceptor->Position - 
			    (pv->vExons+i)->Frame + 1)%3)) %3);
		  
		  /* Evidence exon */
		  (pv->vExons+i)->evidence = 1;

		  /* If strand (-), frame and remainder must be exchanged */
		  if ((pv->vExons+i)->Strand =='-')  
		    { 
		      fraux = (pv->vExons+i)->Frame; 
		      (pv->vExons+i)->Frame = (pv->vExons+i)->Remainder;
		      (pv->vExons+i)->Remainder = fraux;
		    }
		  
		  i = (three)? i+3 : i+1;
		  three = 0;
		  
		  pv->nvSites = pv->nvSites + 2;
	    
		  if (i > NUMEEVIDENCES)
		    printError("Too many evidence exons: Change NUMEEVIDENCES parameter");
		  
		  if (pv->nvSites > NUMSEVIDENCES)
		    printError("Too many evidence sites: Change NUMSEVIDENCES parameter");
		}
	    }
	}
    }
  fclose(file);
  return(i);
}
