/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: ReadGeneModel                                      *
*                                                              *
*   Reading of the rules used at gene construction.            *
***************************************************************/


#include "geneid.h"

long ReadGeneModel (FILE *file, dict *d, int nc[], int ne[], 
		    int UC[][MAXENTRY], int DE[][MAXENTRY], 
          long md[], long Md[], int block[])
{
  char line[MAXLINE];
  char *t1;
  char *line1;
  char *line2;
  char *line3;
  char *line4;
  int a;
  int nlines=0;

  /* GeneModel format is: 
     F1:F2:(...):Fn   F1:F2:(...):Fm dmin:dMax  [block|...] */
  
  while(fgets(line,MAXLINE,file)!=NULL)
    {
      /* For each line extract the features (upstream/downstream), */
      /* the minMax distances and the (optional) blocks */
      if (line[0] !='#')
	{
     /* Split line in four parts: UC DE Dist block */
	  line1 = (char *) strtok(line," ");
	  line2 = (char *) strtok(NULL," ");
	  line3 = (char *) strtok(NULL," "); 
	  line4 = (char *) strtok(NULL," ");

	  /* For each feature in this upstream set */
	  for ( t1 =(char *) strtok(line1,":");
		t1 != NULL;
		t1 = (char *) strtok(NULL, ":") )
	    { 
	      a = setkeyDict(d,t1);
	      UC[a][nc[a]++] = nlines;
	    }
	  
	  /* For each feature in this downstream set */
	  for ( t1 =(char *) strtok(line2,":");
		t1 != NULL;
		t1 = (char *) strtok(NULL, ":") )
	    { 
	      a = setkeyDict(d,t1); 
	      DE[a][ne[a]++]=nlines;
	    } 
	  
	  /* Read the distances */
	  t1 =(char *) strtok(line3,":");
	  md[nlines] = atol(t1); 
	  t1 = (char *) strtok(NULL, ":");
	  Md[nlines] = atol(t1); 
	
     /* Read the block ... if exists */
	  if (line4 != NULL) 
	    {
         block[nlines] = BLOCK;
	    }
	  else 
	    block[nlines] = NONBLOCK;
	  nlines++;
	}
    }
  return(nlines);
}


