/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module:  ReadSequence                                      *
*                                                              *
*   Reading of sequences of DNA in Fasta format                *
***************************************************************/


#include "geneid.h"

extern int VRB;

/* It reads the header of the first DNA sequence (Name) */
int IniReadSequence(FILE* seqfile, char* line)
{
  int res;
  char sAux[MAXSTRING];
  char cAux;

  /* Get locus name */
  res = fscanf(seqfile,"%s",sAux);
  strcpy(line,sAux+1);

  /* Jumping until \n of the first fasta line */
  res = fscanf(seqfile,"%c",&cAux);
  while(cAux != '\n')
    res = fscanf(seqfile,"%c",&cAux);
    
  if (res == EOF)
    printError("Bad format DNA sequence\n");  
  return(res);
}

/* It reads content of current sequence and the header of next */
int ReadSequence (FILE* seqfile, char* Sequence, char* nextLocus)
{
  long pos;
  int res;
  char mess[MAXSTRING];
  char cAux;

  /* fasta format = "atcgata...atta\n" */
  pos = 0;
  res = fscanf(seqfile,"%s\n",Sequence);
  while((res != EOF) && (Sequence[pos] != '>'))
    { 
      pos = pos + strlen(Sequence + pos);
      res = fscanf(seqfile,"%s\n",Sequence + pos);

      if ( VRB && !(pos % 100000) )
	{
	  sprintf(mess, "...%ld bp",pos);
	  printMess(mess);
	}
    }
 
  /* Next Sequence */
  if (res != EOF)
    {
      strcpy(nextLocus,Sequence+pos+1);
      Sequence[pos] = '\0';
      
      /* Jumping until \n of the first fasta line */
      res = fscanf(seqfile,"%c",&cAux);
      while(cAux != '\n')
	res = fscanf(seqfile,"%c",&cAux);
      
      if (res == EOF)
	printError("Bad format DNA sequence\n");  
    }
  return(res);
}

