/***************************************************************
*                                                              *
*       This file is part of the Geneid-3 Distribution         *
*       (c) by Roderic Guigo, Moises Burset and                *
*       Enrique Blanco, 1999                                   *
*                                                              *
*       ScoreExons.c                                           *
*                                                              *
*       This module scores each exon using Markov models       *
***************************************************************/

#include "geneid.h"

/* variable to translate characters to numbers. borrowed from jwf */
int TRANS[] = {
    /* Control characters */     /*  A=0; C=1; G=2; T=3; other = 4  */
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    /* Punctuation and digits */
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    4,4,4,4,4,4,4,4,
    /* Capitals */
    4,0,4,1,4,4,4,2,   /*  @,A-G  */
    4,4,4,4,4,4,4,4,   /*   H-O   */
    4,4,4,4,3,3,4,4,   /*   P-W   */
    4,4,4,4,4,4,4,4,   /* X-Z,etc */
    /* Lower case */
    4,0,4,1,4,4,4,2,   /*  @,A-G  */
    4,4,4,4,4,4,4,4,   /*   H-O   */
    4,4,4,4,3,3,4,4,   /*   P-W   */
    4,4,4,4,4,4,4,4    /* X-Z,etc */
  };

long OligoToInt(char *s, int ls) 
{ 
  long index;
  int weight;
  short i;
  
  index = 0;
  weight = 1;
  
  for ( i=ls-1; i>=0; --i ) 
    {
      index += weight*TRANS[(int)s[i]];
      weight *= 4;
    }
  return(index);
}

void MarkovScan(char *sequence, gparam* gp,
		float *OligoDistIni[3], 
		float *OligoDistTran[3],
		long l1, long l2) 
{
  int OligoLength_1;
  long i;
  int intword;
  short x,cp;
  float previousScore;

  /* Pentanucleotides score */
  for (i=l1; (i<=l2 && *(sequence+i+gp->OligoLength - 1)) ; i++)
    {
      intword=OligoToInt(sequence+i,gp->OligoLength);
      if (intword>=gp->OligoDim)
	for (x=0;x<3;x++)
	  OligoDistIni[x][i-l1] = NULL_OLIGO_SCORE;
      else 
	for (x=0;x<3;x++) 
	  OligoDistIni[x][i-l1] = gp->OligoLogsIni[x][intword];
    }    
  
  /* Hexanucleotides score */
  OligoLength_1=gp->OligoLength+1;
  
  for (x=0; x<FRAMES; x++)
    {
      previousScore = 0.0;
      cp = (3-x) % 3;
      
      for (i=l1; (i<=l2 && *(sequence+i+OligoLength_1 - 1)) ; i++)
	{
	  intword=OligoToInt(sequence+i,OligoLength_1);
	  if (intword>=gp->OligoDim_1)
	    OligoDistTran[x][i-l1]= previousScore + NULL_OLIGO_SCORE;
	  else
	    {
	      OligoDistTran[x][i-l1]=
		previousScore + gp->OligoLogsTran[cp][intword];
	    }
	  previousScore = OligoDistTran[x][i-l1];
	  cp = (cp + 1) % 3;
	}
    }
}

long Score(exonGFF *Exons, long nExons,
	   float *OligoDistIni[3], float *OligoDistTran[3],
	   paramexons* p, int OligoLength, long l1)
{
  long iniexon, endexon, i, j;
  short frame;
  short codonPosition;
  long n=0;
  double scoreMarkov;
  double scoreTotal;
  int OligoLength_1;

  /* hack */  
  long lenexon;
  

  
  OligoLength_1 = OligoLength+1;
  
  for (i=0;i<nExons;i++) 
    {
      /* 1.markov score exon */
      scoreMarkov=0.0;
      iniexon=(Exons+i)->Acceptor->Position - l1;
      frame=(Exons+i)->Frame;
      endexon=(Exons+i)->Donor->Position - l1;
      
      /* hack */
      lenexon=endexon-iniexon+1;
      
      /* Translate frame to position into codon */
      codonPosition = (3 - frame) % 3;

      /* initial probability: Pentanucleotide */
      if (lenexon > OligoLength) 
	{
	  scoreMarkov+=OligoDistIni[codonPosition][iniexon]; 
      
	  /* What kind of adding? */
	  j = (iniexon + (3-codonPosition)) % 3;
	  
	  /* transition probability: Hexanucleotide */    
	  scoreMarkov +=
	    OligoDistTran[j][endexon-OligoLength_1+1] - 
	    OligoDistTran[j][iniexon-1];   
	}

      /* First cutoff */
      if (scoreMarkov >= p->OligoCutoff) 
	{
	  /* 2.score about two sites of exon */
	  scoreTotal = 
	    ((1 - p->OligoWeight) * ((Exons+i)->Acceptor->Score 
				     + (Exons+i)->Donor->Score))
	    + (p->OligoWeight * scoreMarkov); 
	  
	  /* Second cutoff */
	  if (scoreTotal >= p->ExonCutoff) 
	    {
	      Exons[n]=Exons[i];
	      (Exons+n)->Score = scoreTotal  + p->ExonWeight;
	      (Exons+n)->PartialScore = scoreMarkov;
	      n++;
	    }
	}
    }
  return(n);
}


void ScoreExons(char *Sequence, 
	   gparam* gp, packExons* allExons, 
	   long l1, long l2)
{
  long i;
  float *OligoDistIni[3]; 
  float *OligoDistTran[3];
  char mess[MAXSTRING];
  long l;
  
  /* 1. Ready for Markov Scan? */
  printMess("Computing Markov logscore profile");
  
  /* Exons are between l1 and l2 */
  l = l2 - l1 + 1;

  /* allocate memory for markov float arrays of nucleotids */
  if ((OligoDistIni[0] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  
  if ((OligoDistIni[1] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  
  if ((OligoDistIni[2] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  
  if ((OligoDistTran[0] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  
  if ((OligoDistTran[1] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  
  if ((OligoDistTran[2] = 
       (float *) calloc(l, sizeof(float))) == NULL)
    printError("Not enough space to hold oligo log scores");
  /* end memory-markov allocation */
  
  /* Filling OligoDistIni and OligoDistTran */
  MarkovScan(Sequence, gp, OligoDistIni, OligoDistTran, l1, l2);
  
  /* 2. Ready for Score and Filter Exons? */
  printMess("Filtering exons using oligo scores");
  
  allExons->nInitialExons = Score(allExons->InitialExons,
				  allExons->nInitialExons,
				  OligoDistIni,OligoDistTran,
				  gp->Initial,gp->OligoLength, l1);
  sprintf(mess,"Initial Exons \t\t%8ld", allExons->nInitialExons);
  printRes(mess); 
  
  allExons->nInternalExons=Score(allExons->InternalExons,
				 allExons->nInternalExons, 
				 OligoDistIni,OligoDistTran,
				 gp->Internal,gp->OligoLength, l1);
  sprintf(mess,"Internal Exons \t\t%8ld", allExons->nInternalExons);
  printRes(mess); 
  
  allExons->nTerminalExons=Score(allExons->TerminalExons,
				 allExons->nTerminalExons,
				 OligoDistIni,OligoDistTran,
				 gp->Terminal,gp->OligoLength, l1);
  sprintf(mess,"Terminal Exons \t\t%8ld", allExons->nTerminalExons);
  printRes(mess);  
  
  allExons->nSingles=Score(allExons->Singles,
			   allExons->nSingles,
			   OligoDistIni,OligoDistTran,
			   gp->Single,gp->OligoLength, l1);
  sprintf(mess,"Singles \t\t%8ld", allExons->nSingles);
  printRes(mess);

  allExons->nTGAxGTs=Score(allExons->TGAxGT,
			   allExons->nTGAxGTs,
			   OligoDistIni,OligoDistTran,
			   gp->TGAxGT,gp->OligoLength, l1);
  sprintf(mess,"TGAxGTs \t\t%8ld", allExons->nTGAxGTs);
  printRes(mess);

  allExons->nTGAxSTOPs=Score(allExons->TGAxSTOP,
			   allExons->nTGAxSTOPs,
			   OligoDistIni,OligoDistTran,
			   gp->TGAxSTOP,gp->OligoLength, l1);
  sprintf(mess,"TGAxSTOPs \t\t%8ld", allExons->nTGAxSTOPs);
  printRes(mess);
  
  /* Freeing memory(Markov Stats) */  
  for(i=0; i<3; i++)
    {
      free(OligoDistIni[i]);
      free(OligoDistTran[i]);
    }
}


