/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: SortExons                                          *
*                                                              *
*   Sort by acceptor position all predicted exons              *
***************************************************************/


#include "geneid.h"

extern int EVD;

struct exonitem 
{
  exonGFF *Exon;
  struct exonitem *nexitem;
} ExonItem;

void FreeItems(struct exonitem *q)
{
  if (q == NULL)
    ;
  else
    {
      FreeItems(q->nexitem);
      free(q);
    }
}

void cloneExon(exonGFF* e ,exonGFF f)
{


  if ((e->Acceptor = (struct s_site *) malloc(sizeof(struct s_site)))==NULL)
    printError("Not enough space to hold clone site");  

  if ((e->Donor = (struct s_site *) malloc(sizeof(struct s_site)))==NULL)
    printError("Not enough space to hold clone site");  

  e->Acceptor->Position = f.Acceptor->Position;
  e->Acceptor->Score = f.Acceptor->Score;

  e->Donor->Position = f.Donor->Position;  
  e->Donor->Score = f.Donor->Score;
  
  e->Frame = f.Frame;
  e->Strand = f.Strand;
  e->Score = f.Score;
  e->PartialScore = f.PartialScore;
  e->GeneScore = 0;
  e->Remainder = f.Remainder;
  e->Group = f.Group;
  e->evidence = f.evidence;
  e->offset1 = f.offset1;
  e->offset2 = f.offset2;
  e->selected = 0;
}  

void UpdateList(struct exonitem **p, exonGFF* InputExon)
{
  /* Insert at the end if this list */
  while (*p!=NULL)
    p=&((*p)->nexitem); 
  
  /* New item */
  if ((*p= (struct exonitem *) malloc(sizeof(struct exonitem))) == NULL)
    printError("Not enough space to hold one new exonitem");
  
  (*p)->Exon=InputExon;
  (*p)->nexitem=NULL;
}

void SortExons(packExons* allExons, 
	       packExons* allExons_r, 
	       packEvidence* pv,
	       exonGFF* Exons,	       
	       long l1, long l2)
{ 
  struct exonitem **ExonList, *q;
  long i;
  long acceptor;
  long n;
  int offset;
  long l;
  long left;
  long right;

  /* Boundaries of sorting-array */
  left = l1 + COFFSET - LENGTHCODON;
  right = l2 + COFFSET;
  l =  right - left + 1;

  /* Allocate memory for array ExonList */
  if ((ExonList = 
       (struct exonitem **) calloc(l, sizeof(struct exonitem *))) == NULL)
    printError("Not enough space to hold exonitems list");

  for (i=0;i<l;i++)
    ExonList[i]=NULL;

  for (i=0; i<allExons->nInitialExons; i++) 
    {
      acceptor=(allExons->InitialExons+i)->Acceptor->Position - left;
      (allExons->InitialExons+i)->Strand = '+';
      CorrectExon(allExons->InitialExons+i);
      offset = (allExons->InitialExons+i)->offset1;
      UpdateList(&(ExonList[acceptor+offset]), allExons->InitialExons+i); 
    }

  for (i=0;i<allExons->nInternalExons;i++) 
    {
      acceptor=(allExons->InternalExons+i)->Acceptor->Position - left;
      (allExons->InternalExons+i)->Strand = '+';
      CorrectExon(allExons->InternalExons+i);
      offset = (allExons->InternalExons+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons->InternalExons+i); 
    }

  for (i=0;i<allExons->nTerminalExons;i++) 
    {
      acceptor=(allExons->TerminalExons+i)->Acceptor->Position - left;
      (allExons->TerminalExons+i)->Strand = '+';
      CorrectExon(allExons->TerminalExons+i);
      offset = (allExons->TerminalExons+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons->TerminalExons+i); 
    }
  
  for (i=0;i<allExons->nSingles;i++) 
    {
      acceptor=(allExons->Singles+i)->Acceptor->Position - left;
      (allExons->Singles+i)->Strand = '+';
      CorrectExon(allExons->Singles+i);
      offset = (allExons->Singles+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons->Singles+i); 
    }

  for (i=0;i<allExons->nTGAxGTs;i++) 
    {
      acceptor=(allExons->TGAxGT+i)->Acceptor->Position - left;
      (allExons->TGAxGT+i)->Strand = '+';
      CorrectExon(allExons->TGAxGT+i);
      offset = (allExons->TGAxGT+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons->TGAxGT+i); 
    }
  
  for (i=0;i<allExons->nTGAxSTOPs;i++) 
    {
      acceptor=(allExons->TGAxSTOP+i)->Acceptor->Position - left;
      (allExons->TGAxSTOP+i)->Strand = '+';
      CorrectExon(allExons->TGAxSTOP+i);
      offset = (allExons->TGAxSTOP+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons->TGAxSTOP+i); 
    }  
  
  for (i=0;i<allExons_r->nInitialExons;i++) 
    {
      acceptor=(allExons_r->InitialExons+i)->Acceptor->Position - left;
      (allExons_r->InitialExons+i)->Strand = '-';
      CorrectExon(allExons_r->InitialExons+i);
      offset = (allExons_r->InitialExons+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->InitialExons+i); 
    }

  for (i=0;i<allExons_r->nInternalExons;i++) 
    {
      acceptor=(allExons_r->InternalExons+i)->Acceptor->Position - left;
      (allExons_r->InternalExons+i)->Strand = '-';
      CorrectExon(allExons_r->InternalExons+i);
      offset = (allExons_r->InternalExons+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->InternalExons+i); 
    }

  for (i=0;i<allExons_r->nTerminalExons;i++) 
    {   
      acceptor=(allExons_r->TerminalExons+i)->Acceptor->Position - left;
      (allExons_r->TerminalExons+i)->Strand = '-';
      CorrectExon(allExons_r->TerminalExons+i);
      offset = (allExons_r->TerminalExons+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->TerminalExons+i); 
    }

  for (i=0;i<allExons_r->nSingles;i++) 
    {
      acceptor=(allExons_r->Singles+i)->Acceptor->Position - left;
      (allExons_r->Singles+i)->Strand = '-';
      CorrectExon(allExons_r->Singles+i);
      offset = (allExons_r->Singles+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->Singles+i); 
    }

    for (i=0;i<allExons_r->nTGAxGTs;i++) 
    {
      acceptor=(allExons_r->TGAxGT+i)->Acceptor->Position - left;
      (allExons_r->TGAxGT+i)->Strand = '-';
      CorrectExon(allExons_r->TGAxGT+i);
      offset = (allExons_r->TGAxGT+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->TGAxGT+i); 
    }
    
    for (i=0;i<allExons_r->nTGAxSTOPs;i++) 
    {
      acceptor=(allExons_r->TGAxSTOP+i)->Acceptor->Position - left;
      (allExons_r->TGAxSTOP+i)->Strand = '-';
      CorrectExon(allExons_r->TGAxSTOP+i);
      offset = (allExons_r->TGAxSTOP+i)->offset1;
      UpdateList(&(ExonList[acceptor + offset]), allExons_r->TGAxSTOP+i); 
    }
    
  /* Putting evidence exons */
  if (EVD)
    for (i = pv->i1vExons; i < pv->i2vExons ; i++) 
      {  
	acceptor=(pv->vExons + i)->Acceptor->Position - left;
	UpdateList(&(ExonList[acceptor]), pv->vExons+i);
      }

  /* Scanning all the table searching the exons(sorted by acceptor) */
  for (i=0, n=0; i<l; i++) 
    {
      q=ExonList[i];
      while (q != NULL)
	{
	  Exons[n].Acceptor = q->Exon->Acceptor;
	  Exons[n].Donor = q->Exon->Donor;
	  strcpy(Exons[n].Type,q->Exon->Type);
	  Exons[n].Frame = q->Exon->Frame;
	  Exons[n].Strand = q->Exon->Strand;
	  Exons[n].Score = q->Exon->Score;
	  Exons[n].PartialScore = q->Exon->PartialScore;
	  Exons[n].GeneScore = 0; 
	  Exons[n].Remainder = q->Exon->Remainder;
	  Exons[n].Group = q->Exon->Group;
	  Exons[n].evidence = q->Exon->evidence;
	  Exons[n].offset1 = q->Exon->offset1;
	  Exons[n].offset2 = q->Exon->offset2; 
	  Exons[n].selected = 0;

	  if (!(strcmp(Exons[n].Type,sINTERNAL)))
            {
              cloneExon(&Exons[n+1],Exons[n]);
              strcpy(Exons[n+1].Type,sINTERNALGT);
              n++;
            }

          if (!(strcmp(Exons[n].Type,sTERMINAL)))
            {
              cloneExon(&Exons[n+1],Exons[n]);
              strcpy(Exons[n+1].Type,sTERMINALGT);
              n++;
	      }      
	  
	  n++;
	  q=q->nexitem;
	}
      /* Free all chained items in q */
      if (ExonList[i] != NULL)
	FreeItems(ExonList[i]->nexitem); 
    }
  /* Free empty array */
  free(ExonList);
}

   
