/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: geneid                                             *
*                                                              *
*   Main program. Management of the actions of GeneId.         *
***************************************************************/

#include "geneid.h"

/* Setup Flags of GeneId */
int   SFP=0, SDP=0, SAP=0, STP=0, SPP=0,
      EFP=0, EIP=0, ETP=0, EXP=0, ESP=0,EJP=0,EKP=0,
      VRB=0, FWD=1, RVS=1,
      GENAMIC = 1, GENEID = 1, 
      GFF = 0, X10 = 0, 
      EVD = 0;

/* Accounting time and results */
account *m;

/* Management of splice sites prediction and exon building */
void  PredictExons(char *Sequence, long LengthSequence,
         packSites* allSites, packExons* allExons,
         long l1, long l2,
         int Strand, gparam* gp)
{
  char mess[MAXSTRING];
  
  long l1a, l1b,
       l2a, l2b,
       l1c, l2c;

  long cutPoint;

  /* 0. Define boundaries of splice site prediction
        according to current split positions and strand selected */
  if (Strand == FORWARD)
    {
      /* Forward direction */
      /* Starts and Acc boundaries */
      l1a = l1;
      l2a = (l2 == LengthSequence - 1)? l2 : l2 - OVERLAP;

      /* Donors Boundary */
      l1b = l1;
      l2b = l2;

      /* Stops Boundary */
      l1c = l1;
      l2c = l2;

      /* Exons can be built with Stops before cutPoint */
      cutPoint = l1;
    }
  else
    {
      /* Reverse direction */
      /* Starts and Acc Boundaries */
      l1a = l1;
      l2a = l2;

      /* Donor Boundary */
      l1b = (l1 == 0)? l1: l1 + OVERLAP;
      l2b = l2;

      /* Stops boundary */
      l1c = l1;
      l2c = l2;

      /* Exons can be built with Stops before cutPoint */
      cutPoint = (l1 == 0)? l1 : l1 + OVERLAP;
    }

  /* 1. Predicting splice sites of current split of DNA sequence */ 
  printMess ("Computing sites ...");

  allSites->nStartCodons =
    GetSitesWithProfile(Sequence,gp->StartProfile,allSites->StartCodons,l1a,l2a);
  sprintf(mess, "Start Codons \t\t%8ld", allSites->nStartCodons);
  printRes(mess);

  allSites->nAcceptorSites =
    GetSitesWithProfile(Sequence,gp->AcceptorProfile,allSites->AcceptorSites,l1a,l2a);
  sprintf(mess, "Acceptor Sites \t\t%8ld", allSites->nAcceptorSites);
  printRes(mess);

  allSites->nDonorSites =
    GetSitesWithProfile(Sequence,gp->DonorProfile,allSites->DonorSites,l1b,l2b);
  sprintf (mess,"Donor Sites \t\t%8ld", allSites->nDonorSites);
  printRes(mess);

  allSites->nStopCodons =
  GetStopCodons(Sequence,gp->StopProfile, allSites->StopCodons,l1c,l2c);
  sprintf (mess,"Stop Codons \t\t%8ld", allSites->nStopCodons);
  printRes(mess);

  allSites->nPseudoStops =
    GetSitesWithProfile(Sequence,gp->PseudoProfile,allSites->PseudoStops,l1a,l2a);
  sprintf (mess,"PSeudo stops \t\t%8ld", allSites->nPseudoStops);
  printRes(mess);  
  
  /* Total number of predicted splice sites in this strand */
  allSites->nSites = allSites->nStartCodons +
    allSites->nAcceptorSites +
    allSites->nDonorSites +
    allSites->nStopCodons +
    allSites->nPseudoStops;

  sprintf(mess,"---------\t\t%8ld", allSites->nSites);
  printRes(mess);

  /* 2. Building exons with splice sites predicted before */ 
  printMess ("Computing exons ...");   
  
  allExons->nInitialExons =
    BuildInitialExons(allSites->StartCodons,allSites->nStartCodons,
		      allSites->DonorSites,allSites->nDonorSites,
		      allSites->StopCodons,allSites->nStopCodons,
		      gp->MaxDonors,allExons->InitialExons);
  sprintf(mess,"Initial Exons \t\t%8ld", allExons->nInitialExons);
  printRes(mess); 
  
  allExons->nInternalExons =
    BuildInternalExons(allSites->AcceptorSites,allSites->nAcceptorSites,
		       allSites->DonorSites,allSites->nDonorSites,
		       allSites->StopCodons,allSites->nStopCodons,
		       gp->MaxDonors,allExons->InternalExons);
  sprintf(mess,"Internal Exons \t\t%8ld", allExons->nInternalExons);
  printRes(mess); 
  
  allExons->nTerminalExons =
    BuildTerminalExons(allSites->AcceptorSites,allSites->nAcceptorSites,
             allSites->StopCodons,allSites->nStopCodons,
             LengthSequence,allExons->TerminalExons,
             cutPoint);
  sprintf(mess,"Terminal Exons \t\t%8ld", allExons->nTerminalExons);
  printRes(mess); 

  allExons->nSingles =
    BuildSingles(allSites->StartCodons,allSites->nStartCodons,
		 allSites->StopCodons,allSites->nStopCodons,
		 cutPoint, allExons->Singles);
  sprintf(mess,"Single genes \t\t%8ld", allExons->nSingles);
  printRes(mess); 

  allExons->nTGAxGTs = BuildTGAxGT(allSites->PseudoStops, allSites->nPseudoStops, 
				  allSites->DonorSites,allSites->nDonorSites,
				  allSites->StopCodons,allSites->nStopCodons,
				  gp->MaxDonors,allExons->TGAxGT);
  sprintf(mess,"TGAxGT\t\t\t%8ld", allExons->nTGAxGTs);
  printRes(mess);

  allExons->nTGAxSTOPs =
    BuildTGAxSTOP(allSites->PseudoStops, allSites->nPseudoStops,
		 allSites->StopCodons,allSites->nStopCodons,
		 cutPoint, allExons->TGAxSTOP);
  sprintf(mess,"TGAxSTOP\t\t%8ld", allExons->nTGAxSTOPs);
  printRes(mess);  

  /* 3. Scoring and Filtering Exons */
  
  /* Scoring and filtering built exons */
  ScoreExons(Sequence, gp, allExons, l1, l2); 
  
  /* Total number of built exons in this strand */
  allExons->nExons = allExons->nInitialExons + 
    allExons->nInternalExons +
    allExons->nTerminalExons +
    allExons->nSingles +
    allExons->nTGAxGTs +
    allExons->nTGAxSTOPs;

  sprintf(mess,"---------\t\t%8ld", allExons->nExons);
  printRes(mess); 
}

/*****************************************************************************/

int main (int argc, char *argv[])
{
  /* DNA-sequence */
  FILE* seqfile;
  char* Sequence;
  char* RSequence;
  long  LengthSequence;

  /* Boundaries of current split */
  long l1,l2;

  /* Forward Data Structures */
  packSites* allSites;
  packExons* allExons;

  /* Reverse Data Structures */
  packSites* allSites_r;
  packExons* allExons_r;

  /* Sort by acceptor Exons */
  exonGFF* exons; 
  long nExons;

  /* Evidence exons */
  packEvidence* evidence;
  long currVExons=0;

  /* Geneid prediction parameters */
  gparam* gp;
 
  /* Input Files */
  char   SequenceFile[FILENAMELENGTH],
         ExonsFile[FILENAMELENGTH],
         ParamFile[FILENAMELENGTH]="";
  
  /* Name of current sequence */ 
  char Locus[LOCUSLENGTH];
  char nextLocus[LOCUSLENGTH];
  
  char mess[MAXSTRING];

  int lastSplit; 

  /* Best partial predicted genes */
  packGenes* genes;   

  /* Dumpster for back-up operations between splits */
  packDump* dumpster;

  /* Amino-acid dictionary */
  dict* dAA;

  int reading;

  /* 0. Alloc main memory structures */
  Sequence   = (char*) calloc(SEQLENGTH,sizeof(char));
  RSequence  = (char*) calloc(SEQLENGTH,sizeof(char));
  gp         = (gparam *)      RequestMemoryParams();
  allSites   = (packSites*)    RequestMemorySites();
  allSites_r = (packSites*)    RequestMemorySites();
  allExons   = (packExons*)    RequestMemoryExons();
  allExons_r = (packExons*)    RequestMemoryExons();
  exons      = (exonGFF*)      RequestMemorySortExons();
  evidence   = (packEvidence*) RequestMemoryEvidence();
  genes      = (packGenes*)    RequestMemoryGenes();
  dumpster   = (packDump*)     RequestMemoryDumpster();
  dAA        = (dict*)         RequestMemoryAaDictionary();

  /* 1. Read setup options */
  readargv(argc,argv,ParamFile,SequenceFile,ExonsFile);

  /* 2. Read prediction parameters file */
  printMess("Reading parameters..."); 
  readparam(ParamFile, gp);

  /* initializing stats... */
  m = (account*)InitAcc();
  OutputHeader();

  /* 3. Predict Gen or Predict Exons? */
  if (GENEID)
    {
      /* a. Predicting splice sites, exons and genes of DNA sequences */
      
      /* a.0. Reading ADN sequence */
      if ((seqfile = fopen(SequenceFile, "rb"))==NULL) 
        printError("The Sequence file can not be open for read");

      /* reading first sequence (in Fasta format) */
      reading = IniReadSequence(seqfile,Locus);
      while (reading!=EOF)
	{
	  printMess("Reading ADN sequence");
	  reading = ReadSequence(seqfile, Sequence, nextLocus);
	  /* a.1. Processing sequence: Reverse and complement */
	  printMess("Processing ADN sequence");
	  LengthSequence = FetchSequence(Sequence, RSequence);
 	  
	  /* a.2. Reading evidence exons */
	  if (EVD)
	    { 
	      printMess("Reading evidence exons");
	      evidence->nvExons = ReadExonsGFF(ExonsFile, evidence, gp->D); 
	      sprintf(mess,"%ld evidence exons read\n", evidence->nvExons); 
	      printMess(mess);
	    }
	  
	  /* a.3. Split sequence in several parts of the same length */
	  /* l1 is the left boundary and l2 is the right */
	  l1 = 0;
	  l2 = MIN(LENGTHSi-1,LengthSequence-1);
	  lastSplit = (l2 == LengthSequence-1);
	  while((l1 < (LengthSequence - OVERLAP))|| (l1 == 0))
	    {
	      /* a.4. Prediction of sites and exons building */
	      if (FWD) 
		{
		  /* forward sense predictions */ 
		  sprintf(mess,"Running FWD  %s: %ld - %ld", Locus,l1,l2);
		  printMess(mess);
		  PredictExons(Sequence, LengthSequence, 
			       allSites, allExons, l1, l2, FORWARD, gp);
		}      
	      if (RVS) 
		{
		  /* reverse sense predictions */ 
		  sprintf(mess,"Running Reverse  %s: %ld - %ld(%ld - %ld)", 
			  Locus, LengthSequence-1 -l2, 
			  LengthSequence-1 -l1,l1,l2);         
		  printMess(mess);
		  PredictExons(RSequence, LengthSequence,
			       allSites_r, allExons_r, 
			       LengthSequence-1 - l2, 
			       LengthSequence-1 - l1, REVERSE, gp);
		  
		  /* computing sites positions according to forward sense positions */
		  RecomputePositions(allSites_r, LengthSequence);
		  
		  /* exchange acc and donor sites because of Acc < Don needed */
		  SwitchPositions(allExons_r);
		}
	      
	      /* a.5. Sort (forward & reverse) exons by acc position */
	      if (EVD)
		{
		  /* Searching evidence exons in this split */
		  printMess("Searching evidence exons");
		  currVExons = SearchEvidenceExons(evidence, 
				      (lastSplit)?l2:l2-OVERLAP);
		}
	      
	      nExons = allExons->nExons + allExons_r->nExons + currVExons;
	      
	      nExons+= allExons->nInternalExons + allExons_r->nInternalExons +
		allExons->nTerminalExons + allExons_r->nTerminalExons;

	      sprintf(mess,"Sorting %ld Exons by Acceptor\n", nExons);  
	      printMess(mess);
  	      
	      /* Merge predicted exons with some evidence exons */
	      SortExons(allExons, allExons_r, evidence, exons, l1, l2);
	      
	      if (EVD)
		SwitchCounters(evidence);

	      /* a.6. Printing current split predictions (sites and exons) */
	      Output(allSites, allSites_r, allExons, allExons_r, 
		     exons, nExons, Locus, l1, l2, Sequence, gp, dAA); 
	      
	      /* recompute stats about splice sites and exons */
	      updateTotals(m,allSites,allSites_r,allExons,allExons_r);
	      
	      /* a.7. Calling genamic for assembling the best gene */ 
	      if (GENAMIC && nExons)
		{
		  genamic(exons, nExons, genes, gp);
		  printMess("Ending GenAmic...");
		  
		  /* clean hash table of exons */
		  cleanDumpHash(dumpster->h);

		  /* a.8. Backup operations of genes to next split */
		  if (!lastSplit)
		    {
		      /* backup of unused genes */
		      printMess("Backup of d-genes");
		      BackupArrayD(genes, l2 - OVERLAP, gp, dumpster);
		      
		      /* back-up best partial genes */
		      printMess("Backup of partial genes");
		      BackupGenes(genes, gp->nclass, dumpster);
		    }
		}
	      
	      /* a.9. Compute boundaries of next split of sequence */
	      l1 += LENGTHSi - OVERLAP;
	      l2 = MIN(l1 + LENGTHSi -1, LengthSequence-1);
	      lastSplit = (l2 == LengthSequence-1);
	    }/* End of current sequence */
	  
	  if (GENAMIC)
	    {
	      /*  PrintGeneGFF(genes->GOptim,"x","x"); */
	      /*  exit(0); */
	      
	      /* a.10. Printing gene predictions */
	      OutputGene(genes,m->totalExons, Locus, Sequence, gp, dAA);
	      
	      /* reset backup genes data structures to the next DNA sequence */
	      printMess("Cleanning genes");
	      cleanGenes(genes,gp->nclass,dumpster);
	    }
	  
	  /* showing global stats about last sequence predicted */
	  OutputStats(Locus, evidence->nvExons);
	  cleanAcc(m);
	  strcpy(Locus,nextLocus);
   } /* endwhile(reading) */
      
    } /*endifGeneId*/
  else
    {
      /* b. Predicting gens of the exonsGFF using blocks */
      /* It's necessary to increase NUMSITES and NUMEXONS */

      /* b.0. Reading ADN sequence */
      /* open the Sequence File */
      if ((seqfile = fopen(SequenceFile, "rb"))==NULL) 
        printError("The Sequence file can not be open for read");
      
      /* read the sequence (because of translation to proteins) */
      printMess("Reading ADN sequence");
      reading = IniReadSequence(seqfile,Locus);
      if (reading != EOF)
	reading = ReadSequence(seqfile, Sequence, nextLocus);
      
      /* b.1. Reading exons in GFF format */
      printMess("Reading exonsGFF from file");
      nExons = ReadExonsGFF(ExonsFile, evidence, gp->D); 
      
      sprintf(mess,"%ld exons GFF stored\n", nExons); 
      printMess(mess); 
      
      printMess("Executing GenAmic..."); 
      if (nExons) 
	{ 
	  /* b.2. Calling genamic for assembling the best gene */
	  genamic(evidence->vExons, nExons, genes, gp);  
	  printMess("Ending GenAmic..."); 
	}

      PrintGeneGFF(genes->GOptim,"x","x"); 
      exit(0); 
      
      /* b.3. Printing gene predictions */
      OutputGene(genes, nExons, Locus, Sequence, gp, dAA);
    } /* endGenamic */

  /* 4. The End */
  OutputTime(); 
  exit(0);
  return(0);
}
