/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: readargv                                           *
*                                                              *
*   Get setup options and filenames of user input              *
***************************************************************/


#include "geneid.h"

extern int 	SFP,SDP,SAP,STP,SPP,
		EFP,EIP,ETP,EXP,ESP,EJP,EKP,
                VRB,
                FWD,RVS,
                GENEID, GENAMIC,
                GFF, X10,
                EVD;

extern char* optarg;
extern int optind;

char *USAGE="Incorrect usage:\nNAME\n\tgeneid - GeneID, a program to predict genes\nSYNOPSIS\n\tgeneid\t[-bdaepfitjkxs] [-v] [-W] [-C] \n\t\t[-P ParameterFile]\n\t\t[-h] [-X] [-G] [-o] \n\t\t[-O gff_exons_file]\n\t\t[-R gff_evidences-file]\n\t\t<locus_seq_in_fasta_format>\n\n";

void printHelp()
{
  printf("Short Manual for GeneId:\n");
  printf("------------------------\n\n");
  printf("Setup Options:\n\n");
 
  printf("\t-b: Print Start Codons\n");
  printf("\t-d: Print Donor Sites\n ");
  printf("\t-a: Print Acceptor Sites\n");
  printf("\t-e: Print Stop Codons\n");
  printf("\t-p: Print Pseudo Stop Codons(TGA)\n");
  printf("\t-f: Print Initial Exons\n");
  printf("\t-i: Print Internal Exons\n");
  printf("\t-t: Print Terminal Exons\n");
  printf("\t-s: Print Single Genes\n");
  printf("\t-j: Print Exons TGA-GT\n");
  printf("\t-k: Print Exons TGA-STOP\n");

  printf("\t-x: Print All exons\n\n");

  printf("\t-G: Print output predictions in GFF-format\n");
  printf("\t-X: Print extended-format Output Gene Predicted \n\n");

  printf("\t-W: Only Forward Prediction(Watson)\n");
  printf("\t-C: Only Reverse Prediction(Crick)\n");
  printf("\t-o: Only runs exon prediction(not gen prediction)\n");
  printf("\t-O: Only runs gen prediction(not exon prediction)\n\n");
  printf("\t-R: Runs GeneId predictions with evidences-file\n\n");

  printf("\t-P: Change the name of the Parameter File\n");
  printf("\t-v: Verbose. Print all messages\n");
  printf("\t-h: Show this Short Manual\n");
}

void readargv (int argc,char *argv[],
      char *ParamFile, char* SequenceFile,
		char *ExonsFile) 
{
  int c;
  int error=0;
  int geneidOpts = 0;
  int genamicOpts = 0;
  char mess[MAXSTRING];

  /* Reading setup options */
  while ((c = getopt(argc,argv,"oO:bdaepfitjksxXGvR:WCP:h")) != -1)
    switch(c)
      {
      case 'o': GENAMIC--;
	break;
      case 'O': GENEID--;
	strcpy (ExonsFile,optarg);
	break;
      case 'R': EVD++;
	strcpy (ExonsFile,optarg);
	geneidOpts++;
	break;
      case 'b': SFP++;
	geneidOpts++;
	break;
      case 'd': SDP++;
	geneidOpts++;
	break;
      case 'a': SAP++;
	geneidOpts++;
	break;
      case 'e': STP++;
	geneidOpts++;
	break;
      case 'p': SPP++;
	geneidOpts++;
	break;
      case 'f': EFP++;
	geneidOpts++;
	break;
      case 'i': EIP++;
	geneidOpts++;
	break;
      case 't': ETP++;
	geneidOpts++;
	break;
      case 'j': EJP++;
	geneidOpts++;
	break;
      case 'k': EKP++;
	geneidOpts++;
	break;
      case 'x': EXP++;
	geneidOpts++;
	break;
      case 's': ESP++;
	geneidOpts++;
	break;
      case 'v': VRB++; 
	break;
      case 'W': RVS--;
	geneidOpts++;
	break; 
      case 'C': FWD--;
	geneidOpts++;
	break;
      case 'P': strcpy (ParamFile,optarg); 
	break;
      case 'G': GFF++;
	break;
      case 'X': X10++;
   genamicOpts++;
	break;
      case '?':error++;
	break; 
      case 'h': printHelp();
	exit(1);
	break;
      }

  /* Setup Errors: Incompatible options selected */
  if (!GENEID && geneidOpts)
    printError("Incompatible options(-O)");
  
  if (!GENAMIC && genamicOpts)
    printError("Incompatible options(-o)");

  if (!GENAMIC && !GENEID)
    printError("Incompatible options(-o|-O)");

  if (error)
    printError(USAGE);

  /* Setup Errors: Wrong number of filenames */
  /* Get the name of the file *.fasta */
  if (optind < argc)
    {
      strcpy(SequenceFile,argv[optind]);
      optind++;
      if (optind < argc)
	{ 
	  sprintf(mess,"Too many files. Only one filename needed\n%s",USAGE);
	  printError(mess);
	}
    }
  else
    if (GENEID)
      {
	sprintf(mess,"Where is the fasta file(DNA Sequence)?\n%s",USAGE);
	printError(mess);
      }
  
  /* Default parameters file if option P not used */
  if (!strcmp(ParamFile,""))
    strcpy(ParamFile,PARAMETERFILE);
}










