/***************************************************************
*                                                              *
*   This file is part of the Geneid-3 Distribution             *
*   (c) by Enrique Blanco, Moises Burset and                   *
*   Roderic Guigo, 1999                                        *
*                                                              *
*   Module: readparam                                          *
*                                                              *
*   Reading of prediction parameters of GeneId and Gene Model  *
***************************************************************/


#include "geneid.h"

void readparam (char *name, gparam* gp) 
{
  FILE *RootFile;
  char *Geneid;
  char ExternalFileName[FILENAMELENGTH];

  /* oligo sequence and oligo score in HexaLogFile */
  char *word;
  float lscore;       
  int OligoLength_1;
  int i,j,f;
  char mess[MAXSTRING];

  printRes("\n\n\t\t\t** Executing GeneId 3.0 EvoI 2000 imim.es **\n\n");
  Geneid=getenv("GENEID");

  /* 0. Select parameters filename for reading it */
  /* Filename can be: environment option P, env.var GENEID or default */
  /* Using -P option */
  if (strcmp(PARAMETERFILE,name)) 
      {
	sprintf(mess,"Getting param file using -P option: %s",name);
	if ((RootFile = fopen(name,"rb"))==NULL)
	  printError("The parameter file (-P) can not be opened");
      }
  /* Using GENEID environment var */
  else
    if (Geneid)
      {
	sprintf(mess,"Getting param file using GENEID(env var): %s",Geneid);
	sprintf(ExternalFileName,"%s", Geneid);
	if ((RootFile = fopen(ExternalFileName,"rb"))==NULL) 
	  printError("The parameter file (GENEID env.var) can not be opened ");
      }
  /* Using default parameter file */
    else
      {
	sprintf(mess,"Getting param file default");
	if ((RootFile = fopen(name,"rb"))==NULL)
	  printError("The default parameter file can not be opened");
      }
  printMess(mess);

  /* 1. read cutoffs over final score of exons */
  fscanf(RootFile,"%*s\n%f %f %f %f %f %f\n\n",
	 &(gp->Initial->ExonCutoff),
	 &(gp->Internal->ExonCutoff),
	 &(gp->Terminal->ExonCutoff),
	 &(gp->Single->ExonCutoff),
	 &(gp->TGAxGT->ExonCutoff),
	 &(gp->TGAxSTOP->ExonCutoff));

  sprintf(mess,"Exon cutoffs: \t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f",
	  gp->Initial->ExonCutoff,
	  gp->Internal->ExonCutoff,
	  gp->Terminal->ExonCutoff,
	  gp->Single->ExonCutoff,
	  gp->TGAxGT->ExonCutoff,
	  gp->TGAxSTOP->ExonCutoff);
  printMess(mess); 

  /* 2. read cutoffs over Oligonucleotids statistic score of exons */
  fscanf(RootFile,"%*s\n%f %f %f %f %f %f\n\n",     
	 &(gp->Initial->OligoCutoff),
	 &(gp->Internal->OligoCutoff),
	 &(gp->Terminal->OligoCutoff),
	 &(gp->Single->OligoCutoff),
	 &(gp->TGAxGT->OligoCutoff),
	 &(gp->TGAxSTOP->OligoCutoff));

  sprintf(mess,"Oligo cutoffs: \t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f",
	  gp->Initial->OligoCutoff,
	  gp->Internal->OligoCutoff,
	  gp->Terminal->OligoCutoff,
	  gp->Single->OligoCutoff,
	  gp->TGAxGT->OligoCutoff,
	  gp->TGAxSTOP->OligoCutoff);
  printMess(mess); 

  /* 3. read weigths of oligos statistic score */
  fscanf(RootFile,"%*s\n%f %f %f %f %f %f\n\n",
	 &(gp->Initial->OligoWeight),
	 &(gp->Internal->OligoWeight),
	 &(gp->Terminal->OligoWeight),
	 &(gp->Single->OligoWeight),
	 &(gp->TGAxGT->OligoWeight),
	 &(gp->TGAxSTOP->OligoWeight));

  sprintf(mess,"Oligo weights: \t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f",
	  gp->Initial->OligoWeight,
	  gp->Internal->OligoWeight,
	  gp->Terminal->OligoWeight,
	  gp->Single->OligoWeight,
	  gp->TGAxGT->OligoWeight,
	  gp->TGAxSTOP->OligoWeight);
  printMess(mess); 

  /* 4. read weigths to correct the score of exons after cutoff */
  fscanf(RootFile,"%*s\n%f %f %f %f %f %f\n\n",
	 &(gp->Initial->ExonWeight),
	 &(gp->Internal->ExonWeight),
	 &(gp->Terminal->ExonWeight),
	 &(gp->Single->ExonWeight),
	 &(gp->TGAxGT->ExonWeight),
	 &(gp->TGAxSTOP->ExonWeight));

  sprintf(mess,"Exon weights: \t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f\t\t%9.3f",
	  gp->Initial->ExonWeight,
	  gp->Internal->ExonWeight,
	  gp->Terminal->ExonWeight,
	  gp->Single->ExonWeight,
	  gp->TGAxGT->ExonWeight,
	  gp->TGAxSTOP->ExonWeight);
  printMess(mess);

  /* 5. Read splice site profiles */
  /* (a).start codon profile */

  fscanf(RootFile,"%*s\n%*s\n%d %d %f", &(gp->StartProfile->dimension),
    &(gp->StartProfile->offset), &(gp->StartProfile->cutoff));

  for (i=0;i<127;i++) 
    for (j=0;j<PROFILEDIM;j++)
      gp->StartProfile->matrix[i][j]=-INFI;

  for (i=0;i<gp->StartProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StartProfile->matrix['A'][i]));
  for (i=0;i<gp->StartProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StartProfile->matrix['C'][i]));
  for (i=0;i<gp->StartProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StartProfile->matrix['G'][i]));
  for (i=0;i<gp->StartProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StartProfile->matrix['T'][i]));

  printMess("Reading Start Codon Profile\n");
  PrintProfile(stderr, gp->StartProfile);

 /* (b).acceptor site profile */
  
  fscanf(RootFile,"\n%*s\n%d %d %f", &(gp->AcceptorProfile->dimension),
    &(gp->AcceptorProfile->offset), &(gp->AcceptorProfile->cutoff));
  
  for (i=0;i<127;i++)
    for (j=0;j<PROFILEDIM;j++)
      gp->AcceptorProfile->matrix[i][j]=-INFI;

  for (i=0;i<gp->AcceptorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->AcceptorProfile->matrix['A'][i]));
  for (i=0;i<gp->AcceptorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->AcceptorProfile->matrix['C'][i]));
  for (i=0;i<gp->AcceptorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->AcceptorProfile->matrix['G'][i]));
  for (i=0;i<gp->AcceptorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->AcceptorProfile->matrix['T'][i]));

  printMess("Reading Acceptor Profile\n");
  PrintProfile(stderr, gp->AcceptorProfile);  

  /* (c).donor site profile */
  
  fscanf(RootFile,"\n%*s\n%d %d %f", &(gp->DonorProfile->dimension),
    &(gp->DonorProfile->offset), &(gp->DonorProfile->cutoff));
  
  for (i=0;i<127;i++)
    for (j=0;j<PROFILEDIM;j++)
      gp->DonorProfile->matrix[i][j]=-INFI;

  for (i=0;i<gp->DonorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->DonorProfile->matrix['A'][i]));
  for (i=0;i<gp->DonorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->DonorProfile->matrix['C'][i]));
  for (i=0;i<gp->DonorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->DonorProfile->matrix['G'][i]));
  for (i=0;i<gp->DonorProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->DonorProfile->matrix['T'][i]));

  printMess("Reading Donor Profile\n");
  PrintProfile(stderr, gp->DonorProfile);

 /* (d).stop codon profile */
  
  fscanf(RootFile,"\n%*s\n%d %d %f", &(gp->StopProfile->dimension),
    &(gp->StopProfile->offset), &(gp->StopProfile->cutoff));
  for (i=0;i<127;i++)
    for (j=0;j<PROFILEDIM;j++)
      gp->StopProfile->matrix[i][j]=-INFI;

  for (i=0;i<gp->StopProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StopProfile->matrix['A'][i]));
  for (i=0;i<gp->StopProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StopProfile->matrix['C'][i]));
  for (i=0;i<gp->StopProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StopProfile->matrix['G'][i]));
  for (i=0;i<gp->StopProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->StopProfile->matrix['T'][i]));

  printMess("Reading Stop Profile\n");
  PrintProfile(stderr, gp->StopProfile);

 /* (e).Pseudo stop codon profile */
  
  fscanf(RootFile,"\n%*s\n%d %d %f", &(gp->PseudoProfile->dimension),
    &(gp->PseudoProfile->offset), &(gp->PseudoProfile->cutoff));
  for (i=0;i<127;i++)
    for (j=0;j<PROFILEDIM;j++)
      gp->PseudoProfile->matrix[i][j]=-INFI;

  for (i=0;i<gp->PseudoProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->PseudoProfile->matrix['A'][i]));
  for (i=0;i<gp->PseudoProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->PseudoProfile->matrix['C'][i]));
  for (i=0;i<gp->PseudoProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->PseudoProfile->matrix['G'][i]));
  for (i=0;i<gp->PseudoProfile->dimension;i++)
    fscanf(RootFile,"%f", &(gp->PseudoProfile->matrix['T'][i]));

  printMess("Reading Pseudo Profile\n");
  PrintProfile(stderr, gp->PseudoProfile);

  /* 6. read Markov log-likelyhood file */

  fscanf(RootFile,"%*s\n%d", &(gp->OligoLength));

  sprintf(mess,"Oligo word length: %d",gp->OligoLength);
  printMess(mess);

  /* (a). Initial probability matrix */
  fscanf(RootFile,"\n%*s\n");
  
  word=(char *) calloc(gp->OligoLength,sizeof(char));
  gp->OligoDim=(int)pow((float)4,(float)gp->OligoLength);

  sprintf(mess,"Used oligo array size: %ld",gp->OligoDim * 3);
  printMess(mess);

  /* Filling the matrix */
  fscanf(RootFile, "%s %d %d %f", word, &i, &f, &lscore);
  while (*word != '#') 
    { 
      gp->OligoLogsIni[f][i]=lscore;
      fscanf(RootFile, "%s %d %d %f", word, &i, &f, &lscore);
    }
  free(word);
  printMess("Reading Markov Initial probability matrix");

  /* (b). Transition probability matrix */
  fscanf(RootFile,"\n\n%*s\n");

  OligoLength_1= gp->OligoLength + 1;
  word=(char *) calloc(OligoLength_1,sizeof(char));
  gp->OligoDim_1=(int)pow((float)4,(float)OligoLength_1);

  sprintf(mess,"Used oligo array size: %ld",gp->OligoDim_1 * 3);
  printMess(mess);

  /* Filling the matrix */
  fscanf(RootFile, "%s %d %d %f", word, &i, &f, &lscore);
  while (*word != '#') 
    {  
      gp->OligoLogsTran[f][i]=lscore;
      fscanf(RootFile, "%s %d %d %f", word, &i, &f, &lscore);
    }
  free(word);
  printMess("Reading Markov transition probability matrix");

  /* 7. read maximum number of donors per acceptor site */
  fscanf(RootFile,"\n%*s\n%d\n", &(gp->MaxDonors));

  sprintf(mess,"Maximum donors by acceptor = %d", gp->MaxDonors);
  printMess(mess);
  
  /* 8. read the GeneModel */
  fscanf(RootFile,"%*s\n");

  /* Ready to update dictionary of exon types */
  resetDict(gp->D);
  printMess("Dictionary ready");
  
  printMess("Reading GeneModel");
  gp->nclass = ReadGeneModel(RootFile,
			    gp->D, gp->nc,
			    gp->ne,gp->UC,
			    gp->DE,gp->md,
             gp->Md,gp->block);
  sprintf(mess,"%d GM-lines read and stored", gp->nclass);
  printMess(mess);   
}



