/*
  this file is part of the geneid-3 distribution
  (c) by Enrique Blanco, 1999

   ReverseComplement.c
   Reverse and Complement the DNA sequence
*/

#include <stdio.h>

#define SEQLENGTH 4000000
#define MAXSTRING 100

int complement(int c)
{
  switch (c) {
      case 'A':
        return('T');
      case 'C':
        return('G');
      case 'G':
        return('C');
      case 'T':
        return('A');
      case 'a':
        return('t');
      case 'c':
        return('g');
      case 'g':
        return('c');
      case 't':
        return('a');
      default:
        return('N');
      }
}

long ReadSequence (FILE* seqfile, char* Locus, char* Sequence)
{
  long pos;
  int res;
  char cAux;

  /* line has the locus of the current sequence */
  fscanf(seqfile,">%s\n", Locus);

  /* Jumping until \n of the first fasta line */
  res = fscanf(seqfile,"%c",&cAux);
  while(cAux != '\n')
    res = fscanf(seqfile,"%c",&cAux);

  /* fasta format = "atcgata...atta\n" */
  pos = 0;
  res = fscanf(seqfile,"%s\n",Sequence);
  
  /* Only one sequence can be read */
  while(res != EOF) 
    { 
      pos = pos + strlen(Sequence + pos);
      res = fscanf(seqfile,"%s\n",Sequence + pos);

      if ( !(pos % 100000) )
	fprintf(stderr,"...%ld bp\n",pos);
    }

  /* End of sequence */
  pos = pos + strlen(Sequence + pos);

  return(pos);
}

void ReverseComplement (FILE *file1) 
{
  long i,j;
  long LengthSequence;
  char LocusName[MAXSTRING];
  char *Sequence;
  char caux;
  Sequence = (char *) calloc(SEQLENGTH,sizeof(char));

  /* get the sequence */
  LengthSequence = ReadSequence(file1,LocusName,Sequence);
  fprintf(stderr,"ADN sequence: %s || %ld bp\n",
	 LocusName,
	 LengthSequence);
    
  /* reverse and complement the sequence */
  for (i = 0, j = LengthSequence -1; i <= j; i++, j--)
    {     
      caux = Sequence[i]; 
      Sequence[i] = complement(Sequence[j]);
      Sequence[j] = complement(caux);
    }

  /* set the new sequence */
  fprintf(stdout,">%s\n",LocusName);
  for (i = 0; i<LengthSequence; i++)
    { 
      fprintf(stdout,"%c",Sequence[i]);
      if (!((i+1)%60))
	fprintf(stdout,"\n");
    }
}

main(int argc, char *argv[])
{
  FILE *file1;
  char FileName[MAXSTRING];

  /* rc *.fasta -> creates a rc-*.fasta file automatic */

  /* Number of arguments OK */
  if (argc != 2)
    {
      fprintf(stderr,"rc <fasta> \n");
      exit(1);
    }
  
  /* Reading sequence */
  if ((file1=fopen(argv[1], "r"))==NULL)
    {
      fprintf(stderr,"The file %s cannot be open for read\n",argv[1]);
      exit(1);
    }
  
  ReverseComplement(file1);
}












