/*
  this file is part of the geneid-3 distribution
  (c) by Enrique Blanco, 1999

   trad.c
   Reads DNA sequence and print it translation (aminoacids) 
*/

#include <stdio.h>
#include "geneid.h"

int f(char s[])
{
  int i,total;
  
  for(i=0, total=0; i<strlen(s); i++)
    total = (i+1)*s[i] + total;
  total = total % MAXENTRY;

  return(total);
}

void resetDict(dict* d)
{
  int i;

  for(i=0; i<MAXENTRY; i++)
    d->T[i] = NULL;
  d->nextFree = 0;
}

/* Assign a AA-key to the new codon-word and store it */
setAADict(dict *d, char s[], char aA)
{
  node *p;
  node *n;
  int i;
 
  i = f(s);

   /* Alloc the new word */
  if ((n = (node *)malloc(sizeof(node))) == NULL)
      printf("Not enough space to hold a sinonimous\n");
	 
  /* Filling the node */
  strcpy(n->s,s);
  n->key = aA;
  if(d->T[i] == NULL)
	{
	  n->next = NULL;
	  d->T[i] = n;
	}
      else
	{
	  /* There are more nodes in this position: Colission */
	  p = d->T[i];
	  /* Insert at the begining of the list */
	  d->T[i] = n;
	  n->next = p;
	}
}

/* Returns the AA for the codon request; '?' is Not found */ 
char getAADict(dict *d, char s[])
{
  int i;
  int found=0;
  int aa;
  node *p;
  
  i = f(s);
  if(d->T[i]==NULL)
    aa = '?';
  else
    {
      /* There are more nodes in this position */
      p = d->T[i];
      /* Searching until the first position free */
      while( p != NULL && !found )
      {
      if(!strcmp(s,p->s))
         {
            found = 1;
            aa = p->key;
         }
      p = p->next;
      }
      if(!found)
            aa = '?';
    }
  return(aa);
}

dict* RequestMemoryAaDictionary()
{
  dict* dAA;
  
  /* 1. Allocating memory for aminoacid dictionary */
  if ((dAA = (dict *)malloc(sizeof(dict))) == NULL)
    printf("Not enough space to hold Aa-Dictionary\n");

  /* 2. Reset dictionary */
  resetDict(dAA);

  /* 3. Filling it with the aminoacids and the codons */
  setAADict(dAA,"GCA",'A');
  setAADict(dAA,"GCC",'A');
  setAADict(dAA,"GCG",'A');
  setAADict(dAA,"GCT",'A');
  setAADict(dAA,"TGC",'C');
  setAADict(dAA,"TGT",'C');
  setAADict(dAA,"GAC",'D');
  setAADict(dAA,"GAT",'D');
  setAADict(dAA,"GAA",'E');
  setAADict(dAA,"GAG",'E');
  setAADict(dAA,"TTC",'F');
  setAADict(dAA,"TTT",'F');
  setAADict(dAA,"GGA",'G');
  setAADict(dAA,"GGC",'G');
  setAADict(dAA,"GGG",'G');
  setAADict(dAA,"GGT",'G');
  setAADict(dAA,"CAC",'H');
  setAADict(dAA,"CAT",'H');
  setAADict(dAA,"ATA",'I');
  setAADict(dAA,"ATC",'I');
  setAADict(dAA,"ATT",'I');
  setAADict(dAA,"AAA",'K');
  setAADict(dAA,"AAG",'K');
  setAADict(dAA,"TTA",'L');
  setAADict(dAA,"TTG",'L');
  setAADict(dAA,"CTA",'L');
  setAADict(dAA,"CTC",'L');
  setAADict(dAA,"CTG",'L');
  setAADict(dAA,"CTT",'L');
  setAADict(dAA,"ATG",'M');
  setAADict(dAA,"AAC",'N');
  setAADict(dAA,"AAT",'N');
  setAADict(dAA,"CCA",'P');
  setAADict(dAA,"CCC",'P');
  setAADict(dAA,"CCG",'P');
  setAADict(dAA,"CCT",'P');
  setAADict(dAA,"CAA",'Q');
  setAADict(dAA,"CAG",'Q');
  setAADict(dAA,"AGA",'R');
  setAADict(dAA,"AGG",'R');
  setAADict(dAA,"CGA",'R');
  setAADict(dAA,"CGC",'R');
  setAADict(dAA,"CGG",'R');
  setAADict(dAA,"CGT",'R');
  setAADict(dAA,"AGC",'S');
  setAADict(dAA,"AGT",'S');
  setAADict(dAA,"TCA",'S');
  setAADict(dAA,"TCC",'S');
  setAADict(dAA,"TCG",'S');
  setAADict(dAA,"TCT",'S');
  setAADict(dAA,"ACA",'T');
  setAADict(dAA,"ACC",'T');
  setAADict(dAA,"ACG",'T');
  setAADict(dAA,"ACT",'T');
  setAADict(dAA,"GTA",'V');
  setAADict(dAA,"GTC",'V');
  setAADict(dAA,"GTG",'V');
  setAADict(dAA,"GTT",'V');
  setAADict(dAA,"TGG",'W');
  setAADict(dAA,"TAC",'Y');
  setAADict(dAA,"TAT",'Y');
  setAADict(dAA,"TAA",'*');
  setAADict(dAA,"TAG",'*');
  setAADict(dAA,"TGA",'*');

  return(dAA);
}

long XReadSequence (FILE* seqfile, char* Locus, char* Sequence)
{
  long pos;
  int res;

  /* line has the locus of the current sequence */
  fscanf(seqfile,">%s\n", Locus);

  /* fasta format = "atcgata...atta\n" */
  pos = 0;
  res = fscanf(seqfile,"%s\n",Sequence);
  
  /* Only one sequence can be read */
  while(res != EOF) 
    { 
      pos = pos + strlen(Sequence + pos);
      res = fscanf(seqfile,"%s\n",Sequence + pos);

      if ( !(pos % 100000) )
	printf("...%ld bp",pos);
    }

  /* End of sequence */
  pos = pos + strlen(Sequence + pos);

  return(pos);
}

int Translate(long p1, long p2, short fra, short rmd,
              char *s, dict* dAA, char sAux[]) 

{
   int nAA;
   long i;
   char codon[LENGTHCODON+1];

   nAA = 0;
   /* Exon Frame */
   for(i=0; i<fra; i++)
      {
	sAux[i] = s[p1+i] + 32;
	nAA++;
      }

   /* Exon */
   for(i=p1+ fra; i<= p2 - rmd; i=i+3)
   {
      codon[0] = s[i];
      codon[1] = s[i+1];
      codon[2] = s[i+2];
      codon[3] = '\0';

      sAux[nAA] = getAADict(dAA,codon);
      nAA++;
   }

   /* Exon Remainder */
   for(i=p2 - rmd + 1; i <= p2; i++)
   {
      sAux[nAA] = s[i] + 32;
      nAA++;
   }
   sAux[nAA] = '\0';

   /* Corrections about aa not completely built */
   if (fra == 2)
     nAA--;
   if (rmd == 2)
     nAA--;
   
   return(nAA);
}

void TradSequence (FILE *file, long iant,long ipost, int fra)
{
  long i;
  long LengthSequence;
  char LocusName[MAXSTRING];
  char *Sequence;
  char *prot;
  dict* dAA;

  fprintf(stderr,"Asking for memory\n");
  /* Memory */
  Sequence = (char *) calloc(SEQLENGTH,sizeof(char));
  prot = (char *) calloc(SEQLENGTH/LENGTHCODON,sizeof(char));

  fprintf(stderr,"Reading AA dictionary\n");
  /* Dictionary of Aa */
  dAA = (dict*) RequestMemoryAaDictionary();

  fprintf(stderr,"Reading Sequence\n");
  /* Reading sequence */
  LengthSequence = XReadSequence(file,LocusName,Sequence);
  printf("This ADN sequence is %s (%ld nucleotids)\n",
	 LocusName, LengthSequence);
  
  fprintf(stderr,"Translating\n");
  /* Translating */
  Translate(iant, ipost, 
	    fra, 0,
	    Sequence, dAA, 
	    prot); 

  /* Outputs */
  printf("\n> %s \n",
	 LocusName);
  for(i=0; i < strlen(prot); i++)
    {
      printf("%c",prot[i]);
      if (!((i+1)%80))
	printf("\n");
    }
  printf("\n\n");
}

main(int argc, char *argv[])
{
  long iant,ipost;
  FILE *file;
  
  if (argc != 5)
    {
      fprintf(stderr,"ss <.fasta> <pos1> <pos2> <frame>\n");
      exit(1);
    }
  if ((file=fopen(argv[1], "r"))==NULL)
    {
      fprintf(stderr,"The file %s cannot be open for read\n",argv[1]);
      exit(1);
    }
  TradSequence(file,atol(argv[2])-1,atol(argv[3])-1,atoi(argv[4]));
}












